# checks
if [ $(ulimit -n) -lt 500000 ]; then
	echo "ulimit for 'open files' is low - did you run setup.sh?"
	exit 1
fi

if [ $(sysctl net.core.somaxconn|cut -f3 -d' ') -lt 1048576 ]; then
	echo "net.core.somaxconn is low - did you run setup.sh?"
	exit 1
fi

cd $(dirname $0)
currdir=$(pwd)
cd -

# taskset setup
case $(hostname) in

kosi32)
	# threadtest
	coregroup[1]="0"
	for ((i=1;i<32;i++)); do
		coregroup[$(expr $i + 1)]="0-$i"
	done
	wbase=398 # ~1ms work
	gbase=338 # ~1ms work

  # client settings
	reg_client_cores="0-15,32-47"
	max_client_cores="0-15,32-47"
	reg_client_threads=32
	max_client_threads=32

  # server settings
  top_server_core=32
	max_server_threads=16

	# memcached QPS settings
	memcached_conns=40
	memcached_highconns=400
	memcached_qps=800000

	# web settings
	webconns=3000
	;;

kosa64)
	# threadtest
	t="0"
	coregroup[1]="$t"
	for ((i=1;i<32;i++)); do
		t="$t,$(expr $i \* 2)"
		coregroup[$(expr $i + 1)]="$t"
	done
	wbase=378 # ~1ms work
	gbase=282 # ~1ms work

	# client settings
	reg_client_cores="0-31"
	max_client_cores="0-47"
	reg_client_threads=32
	max_client_threads=48

	# server settings
	top_server_core=64
	max_server_threads=32

	# memcached settings
	memcached_conns=25
	memcached_highconns=500
	memcached_qps=500000

	# web settings
	webconns=6000
	;;

*)
	echo "No configuration available for machine $(hostname)"
	exit 1
	;;

esac	

# basedir for result files
basedir=$(dirname $0)/$(hostname)

# common helper (see below)
webmax=$(expr $webconns \* 10)

# repetitions
reps=20

# experiment variants
threadtest_types="arachne arachneS boost fibre go libfiber mordor pthreads qthreads upp"
webserver_types="events fibre go pthreads scope upp"
memcached_types="arachne fibre sharedq pthreads vanilla"

# experiment setups
threadtest_parlevels="1 2 "
threadtest_parlevels+=$(for ((p=4;p<=32;p+=4)); do echo $p; done)
threadtest_model="1 2 4 6 8 10 12 14 16 20 24 28 32 40 48 56 64 80 96 112 128"
threadtest_work="1 4 8 12 16 20 24 28 32 36 40"

webserver_parlevels=$(echo 1 2; for ((p=4;p<=$max_server_threads;p+=4)); do echo $p; done)
webserver_conns=$(for ((c=$webconns;c<=$webmax;c+=$webconns)); do echo $c; done)
#webserver_bigconns=$(for ((c=6000;c<60000;c+=6000)); do echo $c; done)
#webserver_bigconns+=" "
webserver_bigconns=$(for ((c=60000;c<=600000;c+=60000)); do echo $c; done)

memcached_parlevels=$(echo 1 2; for ((p=4;p<=16;p+=4)); do echo $p; done)
memcached_ratios="0.05 0.10 0.15 0.20 0.30 0.40 0.50"

function report_run() {
	ssh mkarsten@linux.cs.uwaterloo.ca $(hostname) $1
	[ $? -eq 77 ] && exit 0
  echo -n "$1 - "; date
}

function kill_it() {
  sig="-15"
  pids=$(pidof $1)
  while [ "$pids" ]; do
    kill $sig $pids
    sleep 1
    sig="-9"
    pids=$(pidof $1)
  done
}
