#!/bin/bash

source $(dirname $0)/config.sh $1
bindir=~/work/fibre/bin
libdir=~/work/fibre/lib
backlog=$(cat /proc/sys/net/ipv4/tcp_max_syn_backlog)
[ -z $backlog ] && exit 0

function run() {
	exp=$1
	type=$2
	rep=$3
	threads=$4
	conns=$5
	updates=$6
	qps=$7

	[ "$type" = "arachne"	] && [ $threads -lt 2 ] && return;
	[ "$type" = "sharedq" ] && [ "$exp" != "exp1" ] && return;

	topcpu=$(expr $top_server_core - 1)
	botcpu=$(expr $top_server_core - $threads)
  client_cores=$max_client_cores
  client_threads=$max_client_threads

	basewl="-s0 -V fb_value -K fb_key -i fb_ia -r 100000"
	case $exp in
		exp1) workload="$basewl -d1";;
		exp3) workload="$basewl -d1";;
		exp4) workload="$basewl -d16";;
		exp5) workload="$basewl -d64"
			client_cores="0-15"
			client_threads=16
			helper_cores="32-47"
			qstep=$(expr $qps / 10);;
		*) return;;
	esac

  # set up executable and file names
  outdir=$basedir/memcached/$exp
  mkdir -p $outdir
  file=$outdir/$type-$threads-$conns-$updates-$qps.$rep
  [ -f $file.out ] && return;
  [ -L $file.out ] && return;

  # handle a special case
	case $type in
		arachne)
      [ $threads -lt 3 ] && maxCores=2 || maxCores=$(expr $threads - 1)
			memcached="$bindir/$type/memcached -t 1 --minNumCores 2 --maxNumCores $maxCores"
			sudo taskset -c $botcpu-$topcpu $bindir/$type/coreArbiterServer --coresUsed $botcpu-$topcpu &
			;;
		*)
			memcached="$bindir/$type/memcached -t $threads"
			;;
	esac

	# report run
	report_run $file

	# start server
	export LD_LIBRARY_PATH=$libdir/$type
	taskset -c $botcpu-$topcpu $memcached -b $backlog -c 32768 -m 10240 -o hashpower=24 &
	# wait a moment...
	sleep 1

	 # start clients, load, then warmup, then run
 	time timeout 30 mutilate $basewl --loadonly
 	sleep 1

 	if [ "$exp" = "exp5" ]; then
		taskset -c $helper_cores \
			mutilate -c$conns -u$updates $workload --noload -w0 -W0 -t600 -T$client_threads -q$qps &
		sleep 3
 		taskset -c $client_cores timeout 600 \
	 		mutilate -c$conns -u$updates $workload --noload -w0 -W0 -t30 -T$(expr $client_threads / 2) --scan=$qstep:$qps:$qstep | tee $file.out
	 	killall mutilate
 	else
		taskset -c $client_cores timeout 60 \
			mutilate -c$conns -u$updates $workload --noload -w0 -W0 -t10 -T$client_threads
		sleep 1
		taskset -c $client_cores timeout 120 \
			perf stat -e task-clock -p $(pidof memcached) -o $file.perf \
			mutilate -c$conns -u$updates $workload --noload -w0 -W0 -t30 -T$client_threads | tee $file.out
	fi

	# kill server
	kill_it memcached
	unset LD_LIBRARY_PATH

  # handle a special case
	case $type in
    arachne)
			while pidof coreArbiterServer >/dev/null 2>&1; do sudo killall coreArbiterServer; sleep 1; done
			;;
	esac
}

# compile memcached without rpath - determine library path via LD_LIBRARY_PATH
for ((r=1;r<=$reps;r+=1)); do
	for t in $memcached_types; do
		for p in $memcached_parlevels; do
			run exp1 $t $r $p $memcached_conns 0.10 0
			run exp1 $t $r $p $memcached_highconns 0.10 0
			run exp4 $t $r $p $memcached_conns 0.10 0
		done
		for u in $memcached_ratios; do
			run exp3 $t $r 16 $memcached_conns $u 0
		done
	done
	for t in arachne fibre localq pthreads vanilla; do
		run exp5 $t $r 16 1 0.10 $memcached_qps
	done
done

exit 0
