#!/bin/bash

source $(dirname $0)/config.sh
bindir=~/work/fibre/apps

function run() {
	exp=$1
	type=$2
	rep=$3
	threads=$4
	time=$5

	case $exp in
		exp1)
			locks=16
			fibres=$threads
			case $type in
				fibre)
					[ $threads -gt 32 ] && threads=32;;
				pthreads)
					;;
				*) return;;
			esac;;
		exp2)
			locks=32
			fibres=$(expr $threads \* $threads);;
		exp3)
			locks=$(expr $(expr $threads + 3) / 4)
			fibres=$(expr $threads \* $threads);;
		*) return;;
	esac

	case $type in
		arachne|arachneS|boost|fibre|libfiber|mordor|pthreads|qthreads|upp)
			work=$(expr $wbase \* $time)
			yield="";;
		go)
			work=$(expr $gbase \* $time)
			yield="-y";;
		*) return;;
	esac

  # set up executable and file names
	prog=$(echo $type|cut -c1)threadtest
	outdir=$basedir/threadtest/$exp
	mkdir -p $outdir
	file=$outdir/$type-$fibres-$time.$rep
	[ -f $file.out ] && return;
	[ -L $file.out ] && return;

  # handle a special case
	[ "$type" = "arachne" -o "$type" = "arachneS" ] && {
		[ $threads -le 3 ] && minCores=1 || minCores=$(expr $threads / 2 - 1)
		[ $threads -le 1 ] && maxCores=1 || maxCores=$(expr $threads - 1)
		[ "$type" = "arachneS" ] && minCores=$maxCores
		addpar="--minNumCores $minCores --maxNumCores $maxCores"
		sudo taskset -c ${coregroup[$threads]} ~/work/fibre/bin/arachne/coreArbiterServer --coresUsed ${coregroup[$threads]} &
		sleep 1
	} || {
		addpar=""
	}

  # report run
  report_run $file

  # start program
	/usr/bin/time -o $file.time -f "user %U\nsyst %S\nreal %e" \
		taskset -c ${coregroup[$threads]} timeout 20 \
		$bindir/$prog $addpar -w $work -u $work -l $locks -t $threads -f $fibres $yield | tee $file.out

  # handle a special case
	[ "$type" = "arachne" -o "$type" = "arachneS" ] && sudo killall coreArbiterServer
}

for ((r=1;r<=$reps;r+=1)); do
	for t in $threadtest_types; do
#		for p in $threadtest_model; do
#			for w in 10 1000; do
#				run exp1 $t $r $p $w
#			done
#		done
		for p in 1 32; do
			for w in $threadtest_work; do
				run exp2 $t $r $p $w # scalability
			done
		done
		for p in $threadtest_parlevels; do
			for w in 20; do
				run exp3 $t $r $p $w # efficiency
			done # work
		done # threads
	done # variants
done # reps

exit 0
