#!/bin/bash
if [ $# -lt 1 ] || [ ! -d $1 ]; then
	echo "$0 <raw-data-dir>"
	exit 1
fi
basedir=$1
mkdir -p data

function process() {
	[ -r $basedir/$2.1.out ] && {
		echo $2 >&2
		rate=$(fgrep -h QPS $basedir/$2.*.out|avg.sh 4|cut -f2,4 -d' ')
		rlat50=$(fgrep -h "read " $basedir/$2.*.out|avg.sh 7|cut -f2,4 -d' ')
		rlat90=$(fgrep -h "read " $basedir/$2.*.out|avg.sh 8|cut -f2,4 -d' ')
		rlat99=$(fgrep -h "read " $basedir/$2.*.out|avg.sh 10|cut -f2,4 -d' ')
		wlat50=$(fgrep -h "update " $basedir/$2.*.out|avg.sh 7|cut -f2,4 -d' ')
		wlat90=$(fgrep -h "update " $basedir/$2.*.out|avg.sh 8|cut -f2,4 -d' ')
		wlat99=$(fgrep -h "update " $basedir/$2.*.out|avg.sh 10|cut -f2,4 -d' ')
		util=$(fgrep -h "utilized " $basedir/$2.*.perf|avg.sh 5|cut -f2,4 -d' ')
		echo $1 $rate $util $rlat50 $rlat90 $rlat99 $wlat50 $wlat90 $wlat99
	}
}

for t in arachne fibre localq sharedq pthreads vanilla; do
	for e in exp1 exp4; do
		for c in 25 40 400 500; do
			[ -r $basedir/memcached/$e/$t-16-$c-0.10-0.1.out ] && \
		  for ((p=1;p<=16;p+=1)); do
				process $p memcached/$e/$t-$p-$c-0.10-0
			done > data/memcached-$e-$t-$c.dat
		done
	done

	for e in exp2; do
		[ -r $basedir/memcached/$e/$t-16-??-0.10-100000.1.out ] && \
		for ((q=100000;q<=1000000;q+=100000)); do
			process $q memcached/$e/$t-16-??-0.10-$q
		done > data/memcached-$e-$t.dat
	done

	for e in exp3; do
		[ -r $basedir/memcached/$e/$t-16-??-0.50-0.1.out ] && \
	  for u in 0.05 0.10 0.15 0.20 0.25 0.30 0.35 0.40 0.45 0.50; do
			process $u memcached/$e/$t-16-??-$u-0
		done > data/memcached-$e-$t.dat
	done

	for e in exp5; do
		for v in 50000 80000; do
			vv=$(expr $v \* 10)
			[ -r $basedir/memcached/$e/$t-16-1-0.10-$vv.1.out ] && \
		  for ((l=2;l<=11;l+=1)); do
				echo -n $(expr $(expr $l - 1) \* $v)
				echo -n ' '
				for f in $basedir/memcached/$e/$t-16-1-0.10-$vv.*.out; do
					tail -n+$l $f|head -1
				done|avg.sh 11|awk '{print $2}'
			done > data/memcached-$e-$t.dat
		done
	done
done
