#!/bin/bash

reps=20
dir=$HOME/tmp/redx10
conns=160

mkdir -p $dir

MEMSERVER=red07
MEMSERVER_IP=10.10.0.7
MEMCLIENT=red01
CLIENTS=red08,red09,red10,red11,red12,red13,red14,red15
MUTCLIENTS=$(for m in $(echo $CLIENTS|tr , ' '); do echo " -a $m";done)

function start_agents() {
    ssh -f white.syn pdsh -w $CLIENTS work/benchmarks/mutilate/mutilate -T 12 -A
}

function stop_agents() {
    ssh white.syn pdsh -w $CLIENTS killall mutilate
}

function warmup() {
  # warm up without rate-limit
  echo $1
  while ! ssh $MEMCLIENT.syn timeout 90 work/benchmarks/mutilate/mutilate --noload \
 -s $MEMSERVER_IP $MUTCLIENTS -K fb_key -V fb_value -i fb_ia -r 1000000 \
 -c $conns -d 1 -t 30 -u 0.03 -Q 1000 -T 12 -C 1 -D 1 > $1; do
    echo trying again...
    stop_agents
    start_agents
    sleep 1
  done
}

function experiment() {

  run=$2
  case $1 in
  vanilla)
    toprate=1000000;;
  vanillaA)
    toprate=1000000;;
  fibre)
    toprate=1000000;;
  fibreA)
    toprate=1000000;;
  pthread)
    [ $conns -le 16 ] && toprate=700000 || toprate=600000;;
  arachne)
    [ $conns -le 16 ] && toprate=900000 || toprate=600000;;
  *)
    echo unknown runtime $1; exit 1;;
  esac

  [ -f $dir/$1.$toprate.$run ] && return

  common="-b 16384 -c 32768 -m 10240 -o hashpower=24"
  case $1 in
  vanilla)
    ssh -f $MEMSERVER.syn taskset -c 0-11 work/memcached/memcached -t 12 $common;;
  vanillaA)
    ssh -f $MEMSERVER.syn taskset -c 0-11 work/memcached/memcachedA -t 12 $common;;
  fibre)
    ssh -f $MEMSERVER.syn taskset -c 0-11 work/fibre/memcached/memcached -t 12 $common;;
  fibreA)
    ssh -f $MEMSERVER.syn taskset -c 0-11 work/fibre/memcached/memcachedA -t 12 $common;;
  pthread)
    ssh -f $MEMSERVER.syn taskset -c 0-11 work/fibre/memcached/memcachedP -t 12 $common;;
  arachne)
    ssh -f $MEMSERVER.syn taskset -c 0-11 sudo work/arachne/CoreArbiter/bin/coreArbiterServer --coresUsed 0-11
    sleep 1
    ssh -f $MEMSERVER.syn work/arachne/memcached/memcached --minNumCores 2 --maxNumCores 11 -t 1 $common;;
  esac

  start_agents

  # load database
  ssh $MEMCLIENT.syn work/benchmarks/mutilate/mutilate --loadonly -s $MEMSERVER_IP -K fb_key -V fb_value -r 1000000

  warmup $dir/$1.warmup.$run
  sleep 1

  # scan rates
  for ((rate=100000;rate<=$toprate;rate+=100000)); do
    file=$dir/$1.$rate.$run
    [ -f $file ] || {
      echo $file
      while ! ssh $MEMCLIENT.syn timeout 90 work/benchmarks/mutilate/mutilate --noload \
 -s $MEMSERVER_IP $MUTCLIENTS -K fb_key -V fb_value -i fb_ia -r 1000000 \
 -c $conns -d 1 -t 30 -u 0.03 -Q 1000 -T 12 -C 1 -D 1 -w 5 -q $rate > $file; do
        echo trying again...
        stop_agents
        start_agents
        sleep 1
        warmup $dir/$1.warmup.$run
        sleep 1
      done
    }
    [ $? -ne 0 ] && break;
    sleep 1
  done

  stop_agents

  case $1 in
  vanilla|fibre)
    ssh $MEMSERVER.syn killall memcached
    ;;
  vanillaA|fibreA)
    ssh $MEMSERVER.syn killall memcachedA
    ;;
  pthread)
    ssh $MEMSERVER.syn killall memcachedP
    ;;
  arachne)
    ssh $MEMSERVER.syn killall memcached
    ssh $MEMSERVER.syn sudo killall coreArbiterServer
    ;;
  *)
    echo unknown runtime $1
    exit 1;;
  esac
}

function cleanup() {
	stop_agents
  ssh $MEMSERVER.syn "killall memcached; killall memcachedA; killall memcachedP"
  ssh $MEMSERVER.syn sudo killall coreArbiterServer
}

trap cleanup EXIT

cleanup

for ((r=1;r<=$reps;r+=1)); do
	for t in arachne fibre pthread fibreA vanilla vanillaA
    experiment $t $r
	done
done

exit 0

# set up manual run
MEMSERVER=red07
MEMSERVER_IP=10.10.0.7
MEMCLIENT=red01
CLIENTS=red08,red09,red10,red11,red12,red13,red14,red15
MUTCLIENTS=$(for m in $(echo $CLIENTS|tr , ' '); do echo " -a $m";done)
#ssh -f $MEMSERVER.syn taskset -c 0-11 work/memcached/memcachedA -t 12 -b 16384 -c 32768 -m 10240 -o hashpower=24
ssh -f $MEMSERVER.syn taskset -c 0-11 work/fibre/memcached/memcachedA -t 12 -b 16384 -c 32768 -m 10240 -o hashpower=24
ssh -f white.syn pdsh -w $CLIENTS work/benchmarks/mutilate/mutilate -A -T 12
ssh $MEMCLIENT.syn work/benchmarks/mutilate/mutilate --loadonly -s $MEMSERVER_IP -K fb_key -V fb_value -r 1000000;\
echo LOADED;\
ssh $MEMCLIENT.syn work/benchmarks/mutilate/mutilate --noload -s $MEMSERVER_IP -K fb_key -V fb_value -i fb_ia -r 1000000 \
 $MUTCLIENTS -c 16 -d 1 -t 30 -u 0.03 -Q 1000 -T 12 -C 1 -D 1;\
ssh $MEMCLIENT.syn work/benchmarks/mutilate/mutilate --noload -s $MEMSERVER_IP -K fb_key -V fb_value -i fb_ia -r 1000000 \
 $MUTCLIENTS -c 16 -d 1 -t 30 -u 0.03 -Q 1000 -T 12 -C 1 -D 1 --scan=200000:1000000:200000 # -q 700000
ssh -f $MEMSERVER.syn killall memcachedA
ssh -f white.syn pdsh -w $CLIENTS killall mutilate

# prepare clients
ssh white.syn pdsh -w $MEMCLIENT,$CLIENTS sudo apt autoremove -y
ssh white.syn pdsh -w $MEMCLIENT,$CLIENTS sudo apt install -y libzmq1
ssh white.syn pdsh -w $MEMCLIENT,$CLIENTS sudo cp ~mkarsten/mkarsten.conf /etc/security/limits.d/mkarsten.conf
ssh white.syn pdsh -w $MEMCLIENT,$CLIENTS sudo reboot
# after reboot: networking limits
ssh white.syn pdsh -w $MEMCLIENT,$CLIENTS sudo sysctl -w net.core.somaxconn=1048576

# prepare $MEMSERVER
sudo apt autoremove -y
sudo apt install -y joe scons gengetopt libzmq-dev # libzmq1 on clients
sudo cp ~mkarsten/mkarsten.conf /etc/security/limits.d/mkarsten.conf
# for many connections with pthread -> reboot afterwards
echo UserTasksMax=infinity >> /etc/systemd/logind.conf
# disable various mitigations in /etc/default/grub
GRUB_CMDLINE_LINUX_DEFAULT="splash quiet pti=off l1tf=off spectre_v2=off spec_store_bypass_disable=off no_stf_barrier noibrs noibpb"
# then
update-grub && reboot

# after reboot: various networking & memory limits
sysctl -w net.core.somaxconn=1048576
sysctl -w net.ipv4.tcp_max_syn_backlog=262144
sysctl -w net.ipv4.tcp_tw_reuse=1
sysctl -w net.ipv4.ip_local_port_range="1024 65535"
sysctl -w vm.max_map_count=2097152

echo 0 > /proc/sys/kernel/yama/ptrace_scope         # gdb attach
echo -1 > /proc/sys/kernel/perf_event_paranoid      # perf anything
echo 0 > /proc/sys/kernel/kptr_restrict             # perf tracing
echo 0 > /proc/sys/kernel/nmi_watchdog              # perf cache tracing

# test RFS on $MEMSERVER
echo 100000 > /proc/sys/net/core/netdev_max_backlog
echo 32768 > /proc/sys/net/core/rps_sock_flow_entries
for f in /sys/class/net/enp130s0/queues/rx-*/rps_flow_cnt; do echo 32768 > $f; done # 16.04
# for f in /sys/class/net/p4p1/queues/rx-*/rps_flow_cnt; do echo 32768 > $f; done # 14.04

# unset RFS on $MEMSERVER
echo 1000 > /proc/sys/net/core/netdev_max_backlog
echo 0 > /proc/sys/net/core/rps_sock_flow_entries
for f in /sys/class/net/enp130s0/queues/rx-*/rps_flow_cnt; do echo 0 > $f; done
# for f in /sys/class/net/p4p1/queues/rx-*/rps_flow_cnt; do echo 0 > $f; done

# change number of RX queues
sudo ethtool -L enp130s0 rx 8

# prepare for arachne on 14.04
sudo mkdir /sys/fs/cgroup/cpuset
sudo mount -t cgroup cpuset -o cpuset /sys/fs/cgroup/cpuset

# install old kernel on 16.04
sudo apt install linux-image-4.4.0-104-generic
sed -i -e 's/GRUB_DEFAULT=0/GRUB_DEFAULT=saved/' /etc/default/grub
sudo grub-reboot '1>Ubuntu, with Linux 4.4.0-104-generic'
reboot



HW Info
=======

mkarsten@red07:~$ cat /proc/cpuinfo
...
processor	: 23
vendor_id	: GenuineIntel
cpu family	: 6
model		: 62
model name	: Intel(R) Xeon(R) CPU E5-2620 v2 @ 2.10GHz
stepping	: 4
microcode	: 0x42e
cpu MHz		: 2400.389
cache size	: 15360 KB
physical id	: 1
siblings	: 12
core id		: 5
cpu cores	: 6
apicid		: 43
initial apicid	: 43
fpu		: yes
fpu_exception	: yes
cpuid level	: 13
wp		: yes
flags		: fpu vme de pse tsc msr pae mce cx8 apic sep mtrr pge mca cmov pat pse36 clflush dts acpi mmx fxsr sse sse2 ss ht tm pbe syscall nx pdpe1gb rdtscp lm constant_tsc arch_perfmon pebs bts rep_good nopl xtopology nonstop_tsc cpuid aperfmperf pni pclmulqdq dtes64 monitor ds_cpl vmx smx est tm2 ssse3 cx16 xtpr pdcm pcid dca sse4_1 sse4_2 x2apic popcnt tsc_deadline_timer aes xsave avx f16c rdrand lahf_lm cpuid_fault epb ssbd ibrs ibpb stibp tpr_shadow vnmi flexpriority ept vpid fsgsbase smep erms xsaveopt dtherm ida arat pln pts md_clear flush_l1d
bugs		: cpu_meltdown spectre_v1 spectre_v2 spec_store_bypass l1tf mds
bogomips	: 4201.38
clflush size	: 64
cache_alignment	: 64
address sizes	: 46 bits physical, 48 bits virtual
power management:

mkarsten@red07:~$ modinfo mlx4_core mlx4_en
filename:       /lib/modules/4.15.0-50-generic/kernel/drivers/net/ethernet/mellanox/mlx4/mlx4_core.ko
version:        4.0-0
license:        Dual BSD/GPL
description:    Mellanox ConnectX HCA low-level driver
author:         Roland Dreier
srcversion:     8E942F74EC6E4F76DBB3AE2
alias:          pci:v000015B3d00001010sv*sd*bc*sc*i*
alias:          pci:v000015B3d0000100Fsv*sd*bc*sc*i*
alias:          pci:v000015B3d0000100Esv*sd*bc*sc*i*
alias:          pci:v000015B3d0000100Dsv*sd*bc*sc*i*
alias:          pci:v000015B3d0000100Csv*sd*bc*sc*i*
alias:          pci:v000015B3d0000100Bsv*sd*bc*sc*i*
alias:          pci:v000015B3d0000100Asv*sd*bc*sc*i*
alias:          pci:v000015B3d00001009sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001008sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001007sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001006sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001005sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001004sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001003sv*sd*bc*sc*i*
alias:          pci:v000015B3d00001002sv*sd*bc*sc*i*
alias:          pci:v000015B3d0000676Esv*sd*bc*sc*i*
alias:          pci:v000015B3d00006746sv*sd*bc*sc*i*
alias:          pci:v000015B3d00006764sv*sd*bc*sc*i*
alias:          pci:v000015B3d0000675Asv*sd*bc*sc*i*
alias:          pci:v000015B3d00006372sv*sd*bc*sc*i*
alias:          pci:v000015B3d00006750sv*sd*bc*sc*i*
alias:          pci:v000015B3d00006368sv*sd*bc*sc*i*
alias:          pci:v000015B3d0000673Csv*sd*bc*sc*i*
alias:          pci:v000015B3d00006732sv*sd*bc*sc*i*
alias:          pci:v000015B3d00006354sv*sd*bc*sc*i*
alias:          pci:v000015B3d0000634Asv*sd*bc*sc*i*
alias:          pci:v000015B3d00006340sv*sd*bc*sc*i*
depends:        devlink
retpoline:      Y
intree:         Y
name:           mlx4_core
vermagic:       4.15.0-50-generic SMP mod_unload 
parm:           debug_level:Enable debug tracing if > 0 (int)
parm:           msi_x:attempt to use MSI-X if nonzero (int)
parm:           num_vfs:enable #num_vfs functions if num_vfs > 0
num_vfs=port1,port2,port1+2 (array of byte)
parm:           probe_vf:number of vfs to probe by pf driver (num_vfs > 0)
probe_vf=port1,port2,port1+2 (array of byte)
parm:           log_num_mgm_entry_size:log mgm size, that defines the num of qp per mcg, for example: 10 gives 248.range: 7 <= log_num_mgm_entry_size <= 12. To activate device managed flow steering when available, set to -1 (int)
parm:           enable_64b_cqe_eqe:Enable 64 byte CQEs/EQEs when the FW supports this (default: True) (bool)
parm:           enable_4k_uar:Enable using 4K UAR. Should not be enabled if have VFs which do not support 4K UARs (default: false) (bool)
parm:           log_num_mac:Log2 max number of MACs per ETH port (1-7) (int)
parm:           log_num_vlan:Log2 max number of VLANs per ETH port (0-7) (int)
parm:           use_prio:Enable steering by VLAN priority on ETH ports (deprecated) (bool)
parm:           log_mtts_per_seg:Log2 number of MTT entries per segment (1-7) (int)
parm:           port_type_array:Array of port types: HW_DEFAULT (0) is default 1 for IB, 2 for Ethernet (array of int)
parm:           enable_qos:Enable Enhanced QoS support (default: off) (bool)
parm:           internal_err_reset:Reset device on internal errors if non-zero (default 1) (int)
filename:       /lib/modules/4.15.0-50-generic/kernel/drivers/net/ethernet/mellanox/mlx4/mlx4_en.ko
version:        4.0-0
license:        Dual BSD/GPL
description:    Mellanox ConnectX HCA Ethernet driver
author:         Liran Liss, Yevgeny Petrilin
srcversion:     386CEBF8D9DAA438F80AE22
depends:        mlx4_core,ptp,devlink
retpoline:      Y
intree:         Y
name:           mlx4_en
vermagic:       4.15.0-50-generic SMP mod_unload 
parm:           udp_rss:Enable RSS for incoming UDP traffic or disabled (0) (uint)
parm:           pfctx:Priority based Flow Control policy on TX[7:0]. Per priority bit mask (uint)
parm:           pfcrx:Priority based Flow Control policy on RX[7:0]. Per priority bit mask (uint)
parm:           inline_thold:Threshold for using inline data (range: 17-104, default: 104) (uint)
